import java.util.Iterator;
import java.util.LinkedList;
/**
 Represents a binary tree of items of type T. Each item is wrapped in a BinaryNode.
*/
class BinaryTree<T> implements Iterable<T>
{
	private BinaryNode<T> root;
	
/**
 Enumeration of allowable iterator types.
*/
	public enum IteratorType
	{
		PreOrder, PostOrder, InOrder, LevelOrder;
	}
	
	private IteratorType iterType = IteratorType.PreOrder;

/**
 Set the iterator type.
*/
	public void setIteratorType( IteratorType t )
	{
		iterType = t;
	}
	
/**
 Create an empty binary tree.
*/
	public BinaryTree()
	{
		this( null );
	}
/**
 Create a binary tree where the root is the input BinaryNode.
*/
	public BinaryTree( BinaryNode<T> n )
	{
		root = n;
	}

/**
 Merge two binary trees as children of item.
*/
	public void merge( T item, BinaryTree<T> t1, BinaryTree<T> t2 )
	{
		if( t1.root == t2.root && t1.root != null )
			throw new IllegalArgumentException();
			
		root = new BinaryNode<T>( item, t1.root, t2.root );
		
		if( this != t1 )
			t1.root = null;
		if( this != t2 )
			t2.root = null;
	}

/**
 Get the size of tree.
*/
	public int getSize()
	{
		if( getRoot() == null ) 
			return 0;
		else
			return getRoot().getSize();
	}

/**
 Get the height of tree.
*/
	public int getHeight()
	{
		if( getRoot() == null )
			return -1;
		else
			return getRoot().getHeight();
	}

/**
 Get the depth of a node.
*/
	public int getDepth(T val)
	{
		if( getRoot() == null )
			return -1;
		else
			return getDepth( getRoot(), val );
	}
	
/**
 Prints items in pre-order.
*/
	public void preOrderPrint()
	{
		preOrderPrint( root, 0);
	}
/**
 Prints items in level-order.
*/
	public void levelOrderPrint()
	{
		// Create stack.
		LinkedList<BinaryNode<T>> qNodes = new LinkedList<BinaryNode<T>>();
		// Push root node onto stack.
		qNodes.offer( getRoot() );
		
		levelOrderPrint( qNodes );
	}
/**
 Create an iterator according to the iterator type. Implements the Iterable interface.
*/
	public Iterator<T> iterator()
	{
		switch( iterType )
		{
			case PostOrder:
				return new BTreePostOrdIterator<T>( this );
			case LevelOrder:
				return new BTreeLevelOrdIterator<T>( this );
			case PreOrder:
				return new BTreePreOrdIterator<T>( this );
			case InOrder:
				return new BTreeInOrdIterator<T>( this );
		}
		return new BTreePreOrdIterator<T>( this );
	}
/**
 Retrieve the root of the binary tree.
*/
	public BinaryNode<T> getRoot() 
	{
		return root;
	}

// Private methods
	
	private void preOrderPrint(  BinaryNode<T> node, int depth )
	{
		T item = node.getItem();
		
		String spaces = " ";
		String formatString = "%" + (6*depth+1) + "s%s\n";
		
		System.out.printf( formatString, spaces, item );
		
		if( node.left != null )
		{
			preOrderPrint( node.left, depth+1 );
		}
		if( node.right != null )
		{
			preOrderPrint( node.right, depth+1 );
		}
	}

	private void levelOrderPrint( LinkedList<BinaryNode<T>> qNodes )
	{
		
		BinaryNode<T> n;
		
		if( !qNodes.isEmpty() )
		{
			// Pop stack.
			n = qNodes.poll();
			System.out.println( n.getItem() );
	
			if( n.left != null )
				// Push left child onto stack.
				qNodes.offer( n.left );

			if( n.right != null )
				// Push right child onto stack.
				qNodes.offer( n.right );

			if( !qNodes.isEmpty() )
				levelOrderPrint( qNodes );
		}	
	}

	private int getDepth(BinaryNode<T> node, T val)
	{
		if( node == null )
			return -1;
		if( node.getItem() == val )
			return 0;
		else
		{
			int left =  getDepth( node.left,  val );
			int right = getDepth( node.right, val ) ;
			
			if( left == -1 && right == -1 )
				return -1;
			
			else
				return 1 + Math.max( left, right );
		}
	}
}